%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%% @doc restapi json
%%% @end
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%%%_* Module declaration ===============================================
%% @private
-module(restapi_json).

%%%_* Exports ==========================================================
%% API
-export([ encode/1
        , decode/1
        , make/2
        ]).

%%%_* API ==============================================================
encode(JSON) ->
  Enc = mochijson2:encoder([{utf8, true}]),
  Enc(JSON).

decode(JSON) ->
  mochijson2:decode(JSON).

make(Req, JSON) when is_binary(JSON) -> maybe_jsonp(Req, JSON);
make(Req, JSON)                      ->
  maybe_jsonp(Req, iolist_to_binary(encode(JSON))).

%%%_* Internal =========================================================
maybe_jsonp(Req, JSON) when is_binary(JSON) ->
  case cowboy_req:qs_val(<<"callback">>, Req) of
    {undefined, _} -> {JSON, Req};
    {CbName, _}    ->
      { iolist_to_binary([CbName, "(", JSON, ");"])
      , cowboy_req:set_resp_header(
          <<"content-type">>,
          <<"application/javascript; charset=utf-8">>, Req)
      }
  end.

%%%_* Emacs ============================================================
%%% Local Variables:
%%% allout-layout: t
%%% erlang-indent-level: 2
%%% End:
