%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%% @doc restapi lager smtp backend
%%% @end
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%%%_* Module declaration ===============================================
%% @private
-module(restapi_lager_smtp_backend).

%%%_* Exports ==========================================================
%% Gen event callbacks
-export([ init/1
        , handle_call/2
        , handle_event/2
        , handle_info/2
        , terminate/2
        , code_change/3
        ]).

%%%_* Records ==========================================================
-record(state, { callback_module
               , smtp_opts
               , from
               , loglevel
               }).

%%%_* Gen events callbacks =============================================
init(Args) ->
  {ok, CbMod}    = tulib_lists:assoc(callback_module, Args),
  {ok, SmtpOpts} = tulib_lists:assoc(smtp_opts, Args),
  {ok, From}     = tulib_lists:assoc(from, Args),
  {ok, Loglevel} = tulib_lists:assoc(loglevel, Args),
  {ok, #state{ callback_module = CbMod
             , smtp_opts       = SmtpOpts
             , from            = From
             , loglevel        = Loglevel
             }}.

handle_call(get_loglevel, #state{loglevel=Loglevel} = State) ->
  {ok, Loglevel, State};
handle_call({set_loglevel, Loglevel}, State) ->
  {ok, ok, State#state{loglevel=Loglevel}}.

%% @private
handle_event({log, Msg},
             #state{ loglevel        = Loglevel
                   , callback_module = CbMod
                   , smtp_opts       = SmtpOpts
                   , from            = From
                   } = State) ->
  case lager_msg:severity_as_int(Msg) =< lager_util:level_to_num(Loglevel) of
    true ->
      Message  = lager_msg:message(Msg),
      Metadata = lager_msg:metadata(Msg),
      Severity = lager_msg:severity(Msg),
      case CbMod:to_addresses(Message, Metadata, Severity) of
        [] -> ok;
        To ->
          Body    = CbMod:format(Message, Metadata, Severity, To),
          {ok, _} = send_email({From, To, Body}, SmtpOpts)
      end;
    false ->
      ok
  end,
  {ok, State};
handle_event(_Event, State) ->
  {ok, State}.

handle_info(_Info, State) ->
  {ok, State}.

terminate(_Reason, _State) ->
  ok.

code_change(_OldVsn, State, _Extra) ->
  {ok, State}.

%%%_* Internal =========================================================
send_email(Email, Opts) ->
  Self = self(),
  Ref  = make_ref(),
  F    = fun (Res) -> Self ! {ok, Ref, Res} end,
  _Pid = gen_smtp_client:send(Email, Opts, F),
  receive
    {ok, Ref, Res} ->
      {ok, _} = Res
  end.

%%%_* Emacs ============================================================
%%% Local Variables:
%%% allout-layout: t
%%% erlang-indent-level: 2
%%% End:
