%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%% @doc restapi monitor alarms
%%% @end
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%%%_* Module declaration ===============================================
-module(restapi_monitor_alarms).

%%%_* Behaviour ========================================================
-behaviour(cowboy_websocket_handler).

%%%_* Exports ==========================================================
-export([ init/3
        , websocket_init/3
        , websocket_handle/3
        , websocket_info/3
        , websocket_terminate/3
        ]).

%%%_* API ==============================================================
init({tcp, http}, _Req, _Opts) ->
  {upgrade, protocol, cowboy_websocket}.

websocket_init(_Any, Req, []) ->
  restapi_lager_backend:subscribe(),
  Reply = cowboy_req:compact(Req),
  {ok, Reply, undefined}.

websocket_handle(_Any, Req, State) ->
  {ok, Req, State, hibernate}.

websocket_info({_From, Severity, Text}, Req, State) ->
  JSON = restapi_json:encode(
           {struct, [ {severity, restapi_type:to_binary(Severity)}
                    , {text, restapi_type:to_binary(Text)}
                    ]}),
  {reply, {text, JSON}, Req, State, hibernate};
websocket_info(_Info, Req, State)                   ->
  {ok, Req, State, hibernate}.

websocket_terminate(_Reason, _Req, _State) ->
  restapi_lager_backend:unsubscribe(),
  ok.

%%%_* Emacs ============================================================
%%% Local Variables:
%%% allout-layout: t
%%% erlang-indent-level: 2
%%% End:
