%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%% @doc restapi monitor metrics
%%% @end
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%%%_* Module declaration ===============================================
-module(restapi_monitor_metrics).

%%%_* Behaviour ========================================================
-behaviour(cowboy_http_handler).

%%%_* Exports ==========================================================
-export([ init/3
        , handle/2
        , terminate/3
        ]).

%%%_* API ==============================================================
init({_Any, http}, Req, []) -> {ok, Req, undefined}.

handle(Req, State) ->
  {Path, _Req} = cowboy_req:path(Req),
  {ok, Reply}  = get_request(Path, Req),
  {ok, Reply, State}.

terminate(_Reason, _Req, _State) -> ok.

%%%_* Internal =========================================================
get_request(Path, Req) ->
  Id0     = lists:last(binary:split(Path, [<<"/">>], [trim, global])),
  Headers =
    [ {<<"content-type">>, <<"application/json; charset=utf-8">>}
    ],
  case metric_exists(Id0) of
    {true, Id} ->
      JSON = restapi_json:encode(get_metric_data(Id)),
      cowboy_req:reply(200, Headers, JSON, Req);
    _          ->
      JSON = restapi_json:encode([{error, nonexistent_metric}]),
      cowboy_req:reply(404, Headers, JSON, Req)
  end.

get_metric_data(Id) ->
  case folsom_metrics:get_metric_info(Id) of
    [{_, [{type, histogram}]}] ->
      [{value, folsom_metrics:get_histogram_statistics(Id)}];
    _                          ->
      [{value, folsom_metrics:get_metric_value(Id)}]
  end.

%% Return true if metric with key 'Id' exists, false otherwise
%% Searches for a metric with 'Id' stored as a binary first and falls
%% back to looking for an existing atom if no matching binary key was
%% found.
metric_exists(Id) when is_list(Id)   -> metric_exists(list_to_binary(Id));
metric_exists(Id) when is_atom(Id)   -> {folsom_metrics:metric_exists(Id), Id};
metric_exists(Id) when is_binary(Id) ->
  case folsom_metrics:metric_exists(Id) of
    true  -> {true, Id};
    false ->
      try metric_exists(erlang:binary_to_existing_atom(Id, utf8))
      catch error:badarg -> {false, Id}
      end
  end.

%%%_* Emacs ============================================================
%%% Local Variables:
%%% allout-layout: t
%%% erlang-indent-level: 2
%%% End:
