%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%% @doc restapi authorities resource
%%% @end
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%%%_* Module declaration ===============================================
%% @private
-module(restapi_resource_authorities).

%%%_* Exports ==========================================================
%% API
-export([ init/3
        , rest_init/2
        , uri_too_long/2
        , allowed_methods/2
        , malformed_request/2
        , is_authorized/2
        , forbidden/2
        , valid_entity_length/2
        , content_types_provided/2
        , charsets_provided/2
        , get_json/2
        , rest_terminate/2
        ]).

%%%_* Includes =========================================================
-include("restapi.hrl").

%%%_* API ==============================================================
init(_Transport, _Req, _Opts) ->
  {upgrade, protocol, cowboy_rest}.

rest_init(Req, Opts) ->
  restapi_resource:rest_init(Req, Opts).

uri_too_long(Req, State) ->
  restapi_resource:uri_too_long(Req, State).

allowed_methods(Req, State) ->
  restapi_resource:allowed_methods(Req, State).

malformed_request(Req, State) ->
  restapi_resource:malformed_request(Req, State).

is_authorized(Req, State) ->
  restapi_resource:is_authorized(Req, State).

forbidden(Req, State) ->
  restapi_resource:forbidden(Req, State).

valid_entity_length(Req, State) ->
  restapi_resource:valid_entity_length(Req, State).

content_types_provided(Req, State) ->
  {[{{<<"application">>, <<"json">>, []}, get_json}], Req, State}.

charsets_provided(Req, State) ->
  {[<<"utf-8">>], Req, State}.

get_json(Req, #state{id = Id0} = State) ->
  Statement = statement(Id0),
  Opts      =
    [ {cache_control, 86400}
    ],
  Id = restapi:prepare(Id0, element(1, Statement)),
  [Authorities, Categories, Subcategories] =
    restapi:run_query(Req, State#state{id = Id, opts = Opts}),
  JSON0 =
    {struct, [ {authorities, restapi:result_to_array(Authorities)}
             , {categories, restapi:result_to_array(Categories)}
             , {subcategories, restapi:result_to_array(Subcategories)}
             ]},
  {JSON, Reply0} = restapi_json:make(Req, JSON0),
  Reply = cowboy_req:set_resp_header(<<"cache-control">>, <<"public">>, Reply0),
  {JSON, Reply, State}.

rest_terminate(Req, State) ->
  restapi_resource:rest_terminate(Req, State).

%%%_* Internal =========================================================
statement(authorities)            ->
  { [ "SELECT myndigheter.id, myndigheter.namn AS name, "
      "myndigheter.subcat AS subcategory "
      "FROM myndigheter ORDER BY subcat;"
    , "SELECT mynd_cat.id, mynd_cat.name FROM mynd_cat "
      "ORDER BY id;"
    , "SELECT mynd_subcat.id, mynd_subcat.sc_name AS name, "
      "mynd_subcat.cat AS category "
      "FROM mynd_subcat ORDER BY id;"
    ]
  , []
  }.

%%%_* Emacs ============================================================
%%% Local Variables:
%%% allout-layout: t
%%% erlang-indent-level: 2
%%% End:
