%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%% @doc restapi smtp callbacks
%%% @end
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%%%_* Module declaration ===============================================
-module(restapi_smtp_callbacks).

%%%_* Exports ==========================================================
-export([ to_addresses/3
        , format/4
        ]).

%%%_* Includes =========================================================
-include_lib("lager/include/lager.hrl").
-include_lib("eunit/include/eunit.hrl").

%%%_* API ==============================================================
to_addresses(_Msg, Metadata, _Severity) ->
  tulib_lists:assoc(alarm_subscribers, Metadata, []).

format(Msg, Metadata, Severity, To) ->
  Subject       = tulib_lists:assoc( subject
                                   , Metadata
                                   , tulib_lists:take(20, Msg)
                                   ),
  Stacktrace    = tulib_lists:assoc(stacktrace, Metadata, undefined),
  ToStr         = string:join(To, ", "),
  Time          = time_str(tulib_lists:assoc( time
                                            , Metadata
                                            , calendar:universal_time()
                                            )),
  FormattedBody = string:join([ "ALARM"
                              , "Severity: " ++ any2str(Severity)
                              , "Time: " ++ Time
                              , "Node: " ++ any2str(node())
                              , ""
                              , Msg
                              , ""
                              , "Stacktrace:"
                              , any2str(Stacktrace)
                              ], "\n"),
  string:join([ tulib_strings:fmt( "Subject: ~p: ~s"
                                 , [Severity, Subject]
                                 )
              , "To: " ++ ToStr
              , ""
              , FormattedBody
              ], "\r\n").

%%%_* Internal =========================================================
any2str(Any) -> tulib_strings:fmt("~p", [Any]).

time_str({{Y, Mo, D},  {H, Mi, S}}) ->
  tulib_strings:fmt("~b-~2..0b-~2..0b ~2..0b:~2..0b:~2..0b",
                    [Y, Mo, D, H, Mi, S]).

%%%_* Tests ============================================================
-ifdef(EUNIT).

format_alarm_test_() ->
  Metadata  = [ {subject, "foobar"}
              , {stacktrace, empty}
              , {time, {{2012, 1, 2}, {3, 4, 5}}}
              ],
  Formatted = string:join([ "Subject: critical: foobar"
                          , "To: a@b.com"
                          , ""
                          , string:join([ "ALARM"
                                        , "Severity: critical"
                                        , "Time: 2012-01-02 03:04:05"
                                        , "Node: " ++ any2str(node())
                                        , ""
                                        , "message"
                                        , ""
                                        , "Stacktrace:"
                                        , "empty"
                                        ], "\n")
                          ], "\r\n"),
  ?_assertEqual( Formatted
               , format("message", Metadata, critical, ["a@b.com"])
               ).

time_test_() ->
  ?_assertEqual("2012-01-02 03:04:05", time_str({{2012, 1, 2}, {3, 4, 5}})).

-endif.

%%%_* Emacs ============================================================
%%% Local Variables:
%%% allout-layout: t
%%% erlang-indent-level: 2
%%% End:
