%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%% @doc restapi util
%%% @end
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%%%_* Module declaration ===============================================
%% @private
-module(restapi_util).

%%%_* Exports ==========================================================
%% API
-export([ fetch/2
        , fetch/3
        , merge/2
        , format_timestamp/1
        , timestamp_to_ms/1
        ]).

%%%_* API ==============================================================
fetch(K, O) when is_atom(K)   -> fetch(atom_to_list(K), O);
fetch(K, O) when is_list(K)   -> fetch(list_to_binary(K), O);
fetch(K, O) when is_binary(K) -> orddict:fetch(K, O).

fetch(K, O, D) -> try fetch(K, O) catch _:_ -> D end.

merge(O1, O2) -> orddict:merge(fun(_K, V1, _V2) -> V1 end, O1, O2).

format_timestamp({_, _, Micro} = Timestamp) ->
  {{Year, Month, Day}, {Hour, Minute, Second}} =
    calendar:now_to_universal_time(Timestamp),
  io_lib:format("~4..0w-~2..0w-~2..0w ~2..0w:~2..0w:~2..0w.~6..0w",
                [Year, Month, Day, Hour, Minute, Second, Micro]).

timestamp_to_ms({MS, S, MiS}) ->
  (MS * 1000000 * 1000) + (S * 1000) + MiS.

%%%_* Emacs ============================================================
%%% Local Variables:
%%% allout-layout: t
%%% erlang-indent-level: 2
%%% End:
