%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%% @doc restapi test suite
%%% @end
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%%%_* Module declaration ===============================================
-module(restapi_SUITE).

%%%_* Exports ==========================================================
%% Test framework
-export([ all/0
        , init_per_suite/1
        , init_per_testcase/2
        , end_per_suite/1
        , end_per_testcase/2
        ]).

%% Test cases
-export([ docs/1
        , docs_jsonp/1
        , docs_unauthorized/1
        , docs_filter/1
        , docs_nin/1
        , docs_nin_filter/1
        , docs_query/1
        , docs_query_filter/1
        , docs_agent/1
        , docs_agent_filter/1
        , docs_x/1
        , docs_x_file/1
        , authorities/1
        , agents/1
        , agents_forbidden/1
        , agents_x/1
        , agents_x_targets/1
        , agents_x_targets_x/1
        , auth_password_grant/1
        , auth_client_credentials_grant/1
        , auth_unauthorized/1
        , uri_too_long/1
        , request_entity_too_large/1
        , user_limit/1
        , client_limit/1
        , developer/1
        ]).

%%%_* Includes =========================================================
-include_lib("common_test/include/ct.hrl").
-include_lib("kernel/include/inet.hrl").
-include_lib("eunit/include/eunit.hrl").

%%%_* Defines ==========================================================
-define(DOC_ID,   "e48e13207341b6bffb7fb1622282247b").
-define(TEST_DIR, filename:join([ code:lib_dir(restapi)
                                , "test/scenario"
                                ])).

%%%_* Test framework ===================================================
all() ->
  [ docs
  , docs_jsonp
  , docs_unauthorized
  , docs_filter
  , docs_nin
  , docs_nin_filter
  , docs_query
  , docs_query_filter
  , docs_agent
  , docs_agent_filter
  , docs_x
  , docs_x_file
  , authorities
  , agents
  , agents_forbidden
  , agents_x
  , agents_x_targets
  , agents_x_targets_x
  , auth_password_grant
  , auth_client_credentials_grant
  , auth_unauthorized
  , uri_too_long
  , request_entity_too_large
  , user_limit
  , client_limit
  , developer
  ].

init_per_suite(Config) ->
  Apps =
    [ sasl
    , ssl
    , crypto
    , compiler
    , syntax_tools
    , inets
    , lager
    , emysql
    , oauth2
    , ranch
    , cowboy
    , restapi
    ],
  {ok, [SysConfig]} =
    file:consult(filename:join([ code:lib_dir(restapi)
                               , "rel/files/sys.config"
                               ])),
  set_config(SysConfig),
  application:set_env(restapi, sql_username, "test"),
  application:set_env(restapi, sql_password, ""),
  application:set_env(restapi, sql_database, "test_piscatus"),
  ensure_apps_started(Apps),
  ok = lhttpc:start(),
  mock(oauth2_token, generate,
       fun() ->
           <<"RHLeJsE5KwpPWF6FMWrXttiISJ6I3WwY">>
       end),
  [{apps, Apps}|Config].

init_per_testcase(TestCase, Config) ->
  restapi_counter:reset(),
  ?MODULE:TestCase({'init', Config}).

end_per_suite(Config) ->
  meck:unload(),
  stop_apps(?config(apps, Config)),
  ok = lhttpc:stop(),
  ok.

end_per_testcase(TestCase, Config) ->
  ?MODULE:TestCase({'end', Config}).

%%%_* Test cases =======================================================
docs({'init', Config}) ->
  run("auth_password_grant"),
  Config;
docs({'end', Config})  -> Config;
docs(_Config)          ->
  run("docs").

docs_jsonp({'init', Config}) ->
  run("auth_password_grant"),
  Config;
docs_jsonp({'end', Config})  -> Config;
docs_jsonp(_Config)          ->
  run("docs_jsonp").

docs_unauthorized({'init', Config}) ->
  run("auth_password_grant"),
  Config;
docs_unauthorized({'end', Config})  -> Config;
docs_unauthorized(_Config)          ->
  run("docs_unauthorized").

docs_filter({'init', Config}) ->
  run("auth_password_grant"),
  Config;
docs_filter({'end', Config})  -> Config;
docs_filter(_Config)          ->
  run("docs_filter").

docs_nin({'init', Config}) ->
  run("auth_password_grant"),
  Config;
docs_nin({'end', Config})  -> Config;
docs_nin(_Config)          ->
  run("docs_nin").

docs_nin_filter({'init', Config}) ->
  run("auth_password_grant"),
  Config;
docs_nin_filter({'end', Config})  -> Config;
docs_nin_filter(_Config)          ->
  run("docs_nin_filter").

docs_query({'init', Config}) ->
  run("auth_password_grant"),
  Config;
docs_query({'end', Config})  -> Config;
docs_query(_Config)          ->
  run("docs_query").

docs_query_filter({'init', Config}) ->
  run("auth_password_grant"),
  Config;
docs_query_filter({'end', Config})  -> Config;
docs_query_filter(_Config)          ->
  run("docs_query_filter").

docs_agent({'init', Config}) ->
  run("auth_password_grant"),
  Config;
docs_agent({'end', Config})  -> Config;
docs_agent(_Config)          ->
  run("docs_agent").

docs_agent_filter({'init', Config}) ->
  run("auth_password_grant"),
  Config;
docs_agent_filter({'end', Config})  -> Config;
docs_agent_filter(_Config)          ->
  run("docs_agent_filter").

docs_x({'init', Config}) ->
  run("auth_password_grant"),
  Config;
docs_x({'end', Config})  -> Config;
docs_x(_Config)          ->
  run("docs_x").

docs_x_file({'init', Config}) ->
  run("auth_password_grant"),
  Config;
docs_x_file({'end', Config})  -> Config;
docs_x_file(_Config)          ->
  run("docs_x_file").

authorities({'init', Config}) ->
  run("auth_password_grant"),
  Config;
authorities({'end', Config})  -> Config;
authorities(_Config)          ->
  run("authorities").

agents({'init', Config}) ->
  run("auth_password_grant"),
  Config;
agents({'end', Config})  -> Config;
agents(_Config)          ->
  run("agents").

agents_forbidden({'init', Config}) ->
  restapi_oauth2_backend:add_client(<<"client1">>, <<"secret123">>),
  run("auth_client_credentials_grant"),
  Config;
agents_forbidden({'end', Config})  ->
  restapi_oauth2_backend:delete_client(<<"client1">>),
  Config;
agents_forbidden(_Config)          ->
  run("agents_forbidden").

agents_x({'init', Config}) ->
  run("auth_password_grant"),
  Config;
agents_x({'end', Config})  -> Config;
agents_x(_Config)          ->
  run("agents_x").

agents_x_targets({'init', Config}) ->
  run("auth_password_grant"),
  Config;
agents_x_targets({'end', Config})  -> Config;
agents_x_targets(_Config)          ->
  run("agents_x_targets").

agents_x_targets_x({'init', Config}) ->
  run("auth_password_grant"),
  Config;
agents_x_targets_x({'end', Config})  -> Config;
agents_x_targets_x(_Config)          ->
  run("agents_x_targets_x").

auth_password_grant({'init', Config}) -> Config;
auth_password_grant({'end', Config})  -> Config;
auth_password_grant(_Config)          ->
  run("auth_password_grant").

auth_client_credentials_grant({'init', Config}) ->
  restapi_oauth2_backend:add_client(<<"client1">>, <<"secret123">>),
  Config;
auth_client_credentials_grant({'end', Config})  ->
  restapi_oauth2_backend:delete_client(<<"client1">>),
  Config;
auth_client_credentials_grant(_Config)          ->
  run("auth_client_credentials_grant").

auth_unauthorized({'init', Config}) -> Config;
auth_unauthorized({'end', Config})  -> Config;
auth_unauthorized(_Config)          ->
  run("auth_unauthorized").

uri_too_long({'init', Config}) -> Config;
uri_too_long({'end', Config})  -> Config;
uri_too_long(_Config)          ->
  run("uri_too_long").

request_entity_too_large({'init', Config}) -> Config;
request_entity_too_large({'end', Config})  -> Config;
request_entity_too_large(_Config)          ->
  run("request_entity_too_large").

user_limit({'init', Config}) ->
  application:set_env(restapi, user_limit, 2),
  run("auth_password_grant"),
  Config;
user_limit({'end', Config})  ->
  application:set_env(restapi, user_limit, 100),
  Config;
user_limit(_Config)          ->
  run("user_limit").

client_limit({'init', Config}) ->
  restapi_oauth2_backend:add_client(<<"client1">>, <<"secret123">>, 2),
  run("auth_client_credentials_grant"),
  Config;
client_limit({'end', Config})  ->
  restapi_oauth2_backend:delete_client(<<"client1">>),
  Config;
client_limit(_Config)          ->
  run("client_limit").

developer({'init', Config}) -> Config;
developer({'end', Config})  -> Config;
developer(_Config)          ->
  run("developer", [{port, 8001}]).

%%%_* Helpers ==========================================================
run(TestName) ->
  run(TestName, [{port, 8000}]).

run(TestName, Params) ->
  Result = katt:run(filename:join([?TEST_DIR, TestName ++ ".apib"]),
                    [{hostname, host()}|Params]),
  ?assertMatch({pass, _, _, _, _}, Result).

host() ->
  [_, Host]     = string:tokens(atom_to_list(node()),"@"),
  {ok, Hostent} = inet:gethostbyname(Host),
  Adresss       = hd(Hostent#hostent.h_addr_list),
  inet_parse:ntoa(Adresss).

ensure_apps_started(App) when is_atom(App)   ->
  ensure_apps_started([App]);
ensure_apps_started(Apps) when is_list(Apps) ->
  lists:foreach(fun do_ensure_apps_started/1, Apps).

stop_apps(Apps) ->
  [ok = application:stop(App) || App <- Apps],
  ok.

do_ensure_apps_started(App) ->
  case application:start(App) of
    ok                                 -> ok;
    {error, {already_started, _}}      -> ok;
    {error, {{already_started, _}, _}} -> ok;
    {error, {not_started, Dep}}        ->
      do_ensure_apps_started(Dep),
      do_ensure_apps_started(App);
    {error, Reason}                    ->
      erlang:throw({start_app_failed, App, Reason})
  end.

mock(Module, Function, MockFun) ->
  ok = meck:new(Module, [no_link, passthrough]),
  ok = meck:expect(Module, Function, MockFun).

set_config(Config) ->
  lists:foreach(
    fun({App, Params}) ->
        [application:set_env(App, K, V) || {K, V} <- Params]
    end, Config).

%%%_* Emacs ============================================================
%%% Local Variables:
%%% allout-layout: t
%%% erlang-indent-level: 2
%%% End:
