%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%% @doc restapi cache tests
%%% @end
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%%%_* Module declaration ===============================================
-module(restapi_cache_tests).

%%%_* Includes =========================================================
-include_lib("eunit/include/eunit.hrl").

%%%_* Defines ==========================================================
-define(TABLE, restapi_cache).

%%%_* Tests ============================================================
restapi_cache_test_() ->
  { foreach
  , fun start/0 % setup
  , fun stop/1  % teardown
  , [ fun test_run/1
    , fun test_ttl/1
    , fun test_reset/1
    ]
  }.

start() -> restapi_cache:start_link().

stop(_Setup) -> exit(whereis(restapi_cache), normal).

test_run(_Setup) ->
  [ ?_assertEqual(true, restapi_cache:run(key, fun() -> {ok, true} end))
  , ?_assertMatch([{_, key, _, true}], ets:lookup(?TABLE, key))
  , ?_assertEqual(true, restapi_cache:run(key, fun() -> false end))
  , ?_assertMatch ([{_, key, _, true}], ets:lookup(?TABLE, key))
  , ?_assertEqual(false, restapi_cache:run(key, fun() -> false end, nocache))
  , ?_assertError(bad_ttl, restapi_cache:run(key, fun() -> {ok, true} end, 1))
  ].

test_ttl(_Setup) ->
  true = restapi_cache:run(key1, fun() -> {ok, true} end, 61),
  true = restapi_cache:run(key2, fun() -> {ok, true} end, 80),
  restapi_cache:clean_cache_loop(?TABLE, ets:first(?TABLE),
                                 calendar:datetime_to_gregorian_seconds(
                                   calendar:local_time()) + 70),
  [ ?_assertEqual([], ets:lookup(?TABLE, key1))
  , ?_assertMatch([{_, key2, _, true}], ets:lookup(?TABLE, key2))
  ].

test_reset(_Setup) ->
  true = restapi_cache:run(key, fun() -> {ok, true} end, 61),
  [ ?_assertEqual(ok, restapi_cache:reset())
  , ?_assertEqual([], ets:tab2list(?TABLE))
  ].

%%%_* Emacs ============================================================
%%% Local Variables:
%%% allout-layout: t
%%% erlang-indent-level: 2
%%% End:
