%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%% @doc restapi counter tests
%%% @end
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%%%_* Module declaration ===============================================
-module(restapi_counter_tests).

%%%_* Includes =========================================================
-include_lib("eunit/include/eunit.hrl").

%%%_* Defines ==========================================================
-define(TABLE, restapi_counter).

%%%_* Tests ============================================================
restapi_counter_test_() ->
  { foreach
  , fun start/0 % setup
  , fun stop/1  % teardown
  , [ fun test_bump/1
    , fun test_ttl/1
    , fun test_reset/1
    ]
  }.

start() -> restapi_counter:start_link().

stop(_Setup) ->
  true = ets:delete_all_objects(?TABLE),
  exit(whereis(restapi_counter), normal).

test_bump(_Setup) ->
  [ ?_assertEqual(0, restapi_counter:get(key1))
  , ?_assertEqual(ok, restapi_counter:bump(key1, 60))
  , ?_assertMatch([{_, key1, _, _}], ets:lookup(?TABLE, key1))
  , ?_assertEqual(1, restapi_counter:get(key1))
  , ?_assertEqual(ok, restapi_counter:bump(key2, 60))
  , ?_assertEqual(1, restapi_counter:get(key2))
  , fun() ->
        [{_, key1, Timestamp, _}] = ets:lookup(?TABLE, key1),
        ok = restapi_counter:bump(key1, 500),
        ?assertMatch([{_, key1, Timestamp, _}], ets:lookup(?TABLE, key1))
    end
  , ?_assertEqual(2, restapi_counter:get(key1))
  , ?_assertEqual(1, restapi_counter:get(key2))
  ].

test_ttl(_Setup) ->
  ok = restapi_counter:bump(key1, 60),
  ok = restapi_counter:bump(key2, 1000),
  restapi_counter:clean_loop(?TABLE, ets:first(?TABLE),
                             calendar:datetime_to_gregorian_seconds(
                               calendar:local_time()) + 70),
  [ ?_assertEqual([], ets:lookup(?TABLE, key1))
  , ?_assertMatch([{_, key2, _, _}], ets:lookup(?TABLE, key2))
  ].

test_reset(_Setup) ->
  ok = restapi_counter:bump(key1, 60),
  ok = restapi_counter:bump(key2, 60),
  [ ?_assertEqual(ok, restapi_counter:reset(key1))
  , ?_assertMatch([{_, key2, _, _}], ets:tab2list(?TABLE))
  ].

%%%_* Emacs ============================================================
%%% Local Variables:
%%% allout-layout: t
%%% erlang-indent-level: 2
%%% End:
