%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%% @doc restapi type tests
%%% @end
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%%%_* Module declaration ===============================================
-module(restapi_type_tests).

%%%_* Includes =========================================================
-include_lib("eunit/include/eunit.hrl").

%%%_* Tests ============================================================
check_test_() ->
  [ ?_assertEqual(
       [ {<<"key1">>, 1.1}
       , {<<"key2">>, 1}
       , {<<"key3">>, <<"x">>}
       , {<<"key4">>, {2012, 11, 11}}
       , {<<"key5">>, {19, 30, 11}}
       , {<<"key6">>, {{2012, 11, 11}, {19, 30, 11}}}
       ],
       restapi_type:check(
         orddict:from_list([ {<<"key1">>, 1.1}
                           , {<<"key2">>, 1}
                           , {<<"key3">>, <<"x">>}
                           , {<<"key4">>, <<"2012-11-11">>}
                           , {<<"key5">>, <<"19:30:11">>}
                           , {<<"key6">>, <<"2012-11-11 19:30:11">>}
                           , {<<"key7">>, <<"xxx">>}
                           ]),
         [ {key1, float}
         , {key2, integer}
         , {key3, string}
         , {key4, date}
         , {key5, time}
         , {key6, datetime}
         ]))
  ].

valid_test_() ->
  [ ?_assert(restapi_type:valid([{key1, integer}, {key2, string}]))
  , ?_assertNot(restapi_type:valid([{"key1", integer}, {key2, string}]))
  , ?_assertNot(restapi_type:valid([{key1, list}, {key2, string}]))
  , ?_assertNot(restapi_type:valid(atom))
  ].

to_list_test_() ->
  [ ?_assertEqual("x",   restapi_type:to_list(x))
  , ?_assertEqual("1",   restapi_type:to_list(1))
  , ?_assertEqual("x",   restapi_type:to_list(<<"x">>))
  , ?_assertEqual("1.1", restapi_type:to_list(1.1))
  , ?_assertEqual("x",   restapi_type:to_list("x"))
  ].

to_binary_test_() ->
  [ ?_assertEqual(<<"x">>,   restapi_type:to_binary(x))
  , ?_assertEqual(<<"1">>,   restapi_type:to_binary(1))
  , ?_assertEqual(<<"x">>,   restapi_type:to_binary(<<"x">>))
  , ?_assertEqual(<<"1.1">>, restapi_type:to_binary(1.1))
  , ?_assertEqual(<<"x">>,   restapi_type:to_binary("x"))
  ].

%%%_* Emacs ============================================================
%%% Local Variables:
%%% allout-layout: t
%%% erlang-indent-level: 2
%%% End:
